@@ -2,13 +2,14 @@ use indexmap::map::{Entry, IndexMap};
 use polars::chunked_array::object::builder::ObjectChunkedBuilder;
 use polars::chunked_array::ChunkedArray;
 
-use bigdecimal::FromPrimitive;
+use bigdecimal::{FromPrimitive, ToPrimitive};
 use chrono::{DateTime, FixedOffset, NaiveDateTime};
 use nu_errors::ShellError;
 use nu_source::{Span, Tag};
 use num_bigint::BigInt;
 use polars::prelude::{
-    DataFrame, DataType, IntoSeries, NamedFrom, ObjectType, PolarsNumericType, Series, TimeUnit,
+    DataFrame, DataType, DatetimeChunked, Int64Type, IntoSeries, NamedFrom, NewChunkedArray,
+    ObjectType, PolarsNumericType, Series,
 };
 use std::ops::{Deref, DerefMut};
 
@@ -74,6 +75,8 @@ pub enum InputType {
     String,
     Boolean,
     Object,
+    Date,
+    Duration,
 }
 
 #[derive(Debug)]
@@ -307,8 +310,8 @@ pub fn create_column(
                 }
             }
         }
-        DataType::Date32 => {
-            let casted = series.date32().map_err(|e| {
+        DataType::Date => {
+            let casted = series.date().map_err(|e| {
                 ShellError::labeled_error(
                     "Casting error",
                     format!("casting error: {}", e),
@@ -344,8 +347,8 @@ pub fn create_column(
 
             Ok(Column::new(casted.name().into(), values))
         }
-        DataType::Date64 => {
-            let casted = series.date64().map_err(|e| {
+        DataType::Datetime => {
+            let casted = series.datetime().map_err(|e| {
                 ShellError::labeled_error(
                     "Casting error",
                     format!("casting error: {}", e),
@@ -381,8 +384,8 @@ pub fn create_column(
 
             Ok(Column::new(casted.name().into(), values))
         }
-        DataType::Time64(timeunit) | DataType::Duration(timeunit) => {
-            let casted = series.time64_nanosecond().map_err(|e| {
+        DataType::Time => {
+            let casted = series.time().map_err(|e| {
                 ShellError::labeled_error(
                     "Casting error",
                     format!("casting error: {}", e),
@@ -395,14 +398,7 @@ pub fn create_column(
                 .skip(from_row)
                 .take(size)
                 .map(|v| match v {
-                    Some(a) => {
-                        let nanoseconds = match timeunit {
-                            TimeUnit::Second => a / 1_000_000_000,
-                            TimeUnit::Millisecond => a / 1_000_000,
-                            TimeUnit::Microsecond => a / 1_000,
-                            TimeUnit::Nanosecond => a,
-                        };
-
+                    Some(nanoseconds) => {
                         let untagged = if let Some(bigint) = BigInt::from_i64(nanoseconds) {
                             UntaggedValue::Primitive(Primitive::Duration(bigint))
                         } else {
@@ -495,7 +491,7 @@ pub fn insert_row(column_values: &mut ColumnMap, dictionary: Dictionary) -> Resu
 // The names for the columns are the enumerated numbers from the values
 pub fn insert_table(column_values: &mut ColumnMap, table: Vec<Value>) -> Result<(), ShellError> {
     for (index, value) in table.into_iter().enumerate() {
-        let key = format!("{}", index);
+        let key = index.to_string();
         insert_value(value, key, column_values)?;
     }
 
@@ -528,6 +524,12 @@ pub fn insert_value(
             UntaggedValue::Primitive(Primitive::Boolean(_)) => {
                 col_val.column_type = Some(InputType::Boolean);
             }
+            UntaggedValue::Primitive(Primitive::Date(_)) => {
+                col_val.column_type = Some(InputType::Date);
+            }
+            UntaggedValue::Primitive(Primitive::Duration(_)) => {
+                col_val.column_type = Some(InputType::Duration);
+            }
             _ => col_val.column_type = Some(InputType::Object),
         }
         col_val.values.push(value);
@@ -550,6 +552,14 @@ pub fn insert_value(
             | (
                 UntaggedValue::Primitive(Primitive::Boolean(_)),
                 UntaggedValue::Primitive(Primitive::Boolean(_)),
+            )
+            | (
+                UntaggedValue::Primitive(Primitive::Date(_)),
+                UntaggedValue::Primitive(Primitive::Date(_)),
+            )
+            | (
+                UntaggedValue::Primitive(Primitive::Duration(_)),
+                UntaggedValue::Primitive(Primitive::Duration(_)),
             ) => col_val.values.push(value),
             _ => {
                 col_val.column_type = Some(InputType::Object);
@@ -600,13 +610,40 @@ pub fn from_parsed_columns(
                     let mut builder =
                         ObjectChunkedBuilder::<Value>::new(&name, column.values.len());
 
-                    for v in column.values.iter() {
+                    for v in &column.values {
                         builder.append_value(v.clone());
                     }
 
                     let res = builder.finish();
                     df_series.push(res.into_series())
                 }
+                InputType::Date => {
+                    let it = column.values.iter().map(|v| {
+                        if let UntaggedValue::Primitive(Primitive::Date(date)) = &v.value {
+                            Some(date.timestamp_millis())
+                        } else {
+                            None
+                        }
+                    });
+
+                    let res: DatetimeChunked =
+                        ChunkedArray::<Int64Type>::new_from_opt_iter(&name, it).into();
+
+                    df_series.push(res.into_series())
+                }
+                InputType::Duration => {
+                    let it = column.values.iter().map(|v| {
+                        if let UntaggedValue::Primitive(Primitive::Duration(duration)) = &v.value {
+                            Some(duration.to_i64().expect("Not expecting NAN in duration"))
+                        } else {
+                            None
+                        }
+                    });
+
+                    let res = ChunkedArray::<Int64Type>::new_from_opt_iter(&name, it);
+
+                    df_series.push(res.into_series())
+                }
             }
         }
     }
@@ -615,12 +652,10 @@ pub fn from_parsed_columns(
 
     match df {
         Ok(df) => Ok(NuDataFrame::new(df)),
-        Err(e) => {
-            return Err(ShellError::labeled_error(
-                "Error while creating dataframe",
-                format!("{}", e),
-                span,
-            ))
-        }
+        Err(e) => Err(ShellError::labeled_error(
+            "Error while creating dataframe",
+            e.to_string(),
+            span,
+        )),
     }
 }
